@@ -3,7 +3,7 @@
 import os
 import re
 
-from flask import (Blueprint, render_template, current_app, request, Response,
+from flask import (Blueprint, render_template, request, Response,
                    flash, url_for, redirect, safe_join, make_response,
                    send_file, send_from_directory)
 from flaskext.babel import gettext as _
@@ -11,22 +11,44 @@ import json
 
 from contextlib import closing
 
-from whoosh.index import open_dir
-from whoosh.qparser import MultifieldParser
-from .whoosh_multi_field_spelling_correction import MultiFieldQueryCorrector
-
 from .extensions import db
 from gutenberg_models import (GutenbergBook, GutenbergFile,
                               GutenbergCreator, gutenberg_books_creator_map)
+from gutenberg_content import find_htmlz, find_epub
 from config import config
 
-import pagination_helper
+from whoosh_search import paginated_search
+from whoosh.index import open_dir
 from .endpoint_description import EndPointDescription
 
+DEFAULT_RESULTS_PER_PAGE = 20
+DEFAULT_SEARCH_COLUMNS = ['title', 'creator', 'contributor']  # names correspond to fields in whoosh schema
+
 gutenberg = Blueprint('gutenberg', __name__, url_prefix='/books')
 etext_regex = re.compile(r'^etext(\d+)$')
-DEFAULT_SEARCH_COLUMNS = ['title', 'creator', 'contributor']  # names correspond to fields in whoosh schema
-DEFAULT_RESULTS_PER_PAGE = 20
+
+
+flask_app = None
+is_init = False
+
+
+def set_flask_app(app):
+    global flask_app
+    flask_app = app
+
+
+def init_db():
+    global is_init
+    global flask_app
+    if not is_init:
+        if flask_app is None:
+            raise Exception("init_db called when flask app not set")
+        # set global config variables referenced by SQLAlchemy
+        flask_app.config['SQLALCHEMY_ECHO'] = config().getboolean('GUTENBERG', 'sqlalchemy_echo')
+        database_path = config().get_path('GUTENBERG', 'sqlalchemy_database_uri')
+        flask_app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite:///' + os.path.abspath(database_path)
+        db.init_app(flask_app)
+        is_init = True
 
 
 @gutenberg.route('/')
@@ -39,12 +61,16 @@ def search():
     query = request.args.get('q', '').strip()
     pagination = None
     if query:
+        index_dir = config().get_path('GUTENBERG', 'index_dir')
         page = int(request.args.get('page', 1))
-        (pagination, suggestion) = paginated_search(query, page)
+        (pagination, suggestion) = paginated_search(index_dir, DEFAULT_SEARCH_COLUMNS, query, page, sort_column='creator')
     else:
         flash(_('Please input keyword(s)'), 'error')
     #print pagination.items
-    return render_template('gutenberg/search.html', pagination=pagination, keywords=query, suggestion=suggestion, fn_author_to_query=author_to_query, endpoint_desc=EndPointDescription('gutenberg.search', None))
+    return render_template('gutenberg/search.html', pagination=pagination,
+                           keywords=query, suggestion=suggestion, fn_author_to_query=author_to_query,
+                           endpoint_desc=EndPointDescription('gutenberg.search', None),
+                           files_exist=files_exist)
 
 
 @gutenberg.route('/mirror/<path:filename>')
@@ -67,68 +93,6 @@ def author_to_query(author):
     return u'creator:"{0}" OR contributor:"{0}"'.format(author)
 
 
-def paginated_search(query_text, page=1, pagelen=DEFAULT_RESULTS_PER_PAGE):
-    """
-    Return a tuple consisting of an object that emulates an SQLAlchemy pagination object and corrected query suggestion
-    pagelen specifies number of hits per page
-    page specifies page of results (first page is 1)
-    """
-    index_dir = config().get_path('GUTENBERG', 'index_dir')
-    query_text = unicode(query_text)  # Must be unicode
-    ix = open_dir(index_dir)
-    sort_column = 'creator'
-    with ix.searcher() as searcher:
-        query = MultifieldParser(DEFAULT_SEARCH_COLUMNS, ix.schema).parse(query_text)
-        try:
-            # search_page returns whoosh.searching.ResultsPage
-            results = searcher.search_page(query, page, pagelen=pagelen, sortedby=sort_column)
-            total = results.total
-        except ValueError:  # Invalid page number
-            results = []
-            total = 0
-        paginate = pagination_helper.Pagination(page, pagelen, total, [dict(r.items()) for r in results])
-        corrections = deduplicate_corrections(get_query_corrections(searcher, query, query_text))  # list of Corrector objects
-
-        #hf = whoosh.highlight.HtmlFormatter(classname="change")
-        #html = corrections.format_string(hf)
-        return (paginate, [c.string for c in corrections])
-
-
-def deduplicate_corrections(corrections):
-    """
-    Return list of correction that omits entries where the query is unmodified
-    :param corrections: list of Corrector objects
-    :returns: list of Corrector objects
-    """
-    # Using values from a dictionary comprehension rather than a list comprehension in order to deduplicate
-    #return {c.string : c for c in corrections if c.original_query != c.query}.values()
-    # We can't use dictionary comprehension because we are stuck on python 2.6 for Debian stable
-    return dict((c.string, c) for c in corrections if c.original_query != c.query).values()
-
-
-def get_query_corrections(searcher, query, qstring):
-    """
-    Suggest alternate spelling for search terms by searching each column with
-    spelling correction support in turn.
-
-    :param searcher: whoosh searcher object
-    :param query: whoosh query object
-    :param qstring: search string that was passed to the query object
-    :returns: MultiFieldQueryCorrector with one corrector for each corrected column
-    """
-    fieldnames = [name for name, field in searcher.schema.items() if field.spelling]
-    correctors = {}
-    for fieldname in fieldnames:
-        if fieldname not in correctors:
-            correctors[fieldname] = searcher.corrector(fieldname)
-    terms = []
-    for token in query.all_tokens():
-        if token.fieldname in correctors:
-            terms.append((token.fieldname, token.text))
-
-    return MultiFieldQueryCorrector(correctors, terms, prefix=2, maxdist=1).correct_query(query, qstring)
-
-
 @gutenberg.route('/titles')
 def by_title():
     page = int(request.args.get('page', 1))
@@ -166,6 +130,28 @@ def mirror_exists(file_rec):
     return os.path.exists(path)
 
 
+def textId2number(textId):
+    return int(textId[5:])
+
+
+def files_exist(textId, record=None):
+    """Returns true if any files exist for this e-text
+    in the local dataset"""
+    if record is None:
+        record = GutenbergBook.query.filter_by(textId=textId).first()
+    if record is None:
+        return False
+    for x in record.gutenberg_files:
+        if mirror_exists(x):
+            return True
+    pgid = textId2number(textId)
+    if find_htmlz(pgid) is not None:
+        return True
+    if find_epub(pgid) is not None:
+        return True
+    return False
+
+
 @gutenberg.route('/text/<textId>/details')
 def text(textId):
     # Profiling results showing occasional lags.
@@ -179,6 +165,16 @@ def text(textId):
             print "WARNING: Gutenberg file " + mirror_path(x.file) + " not found"
     record.gutenberg_files = filter(mirror_exists, record.gutenberg_files)
 
+    pgid = textId2number(textId)
+    if find_htmlz(pgid) is not None:
+        htmlz_url = url_for('gutenberg_content_views.htmlz_index', pgid=pgid)
+    else:
+        htmlz_url = None
+    if find_epub(pgid) is not None:
+        epub_url = url_for('gutenberg_content_views.epub_ext', pgid=pgid)
+    else:
+        epub_url = None
+
     # fields format is list of tuples:
     # (Table row heading for display, gutenberg_books col name,
     #  sub-table col name if applicable)
@@ -190,7 +186,8 @@ def text(textId):
         (_('Category'), 'gutenberg_categories', 'category'),
         (_('Language'), 'gutenberg_languages', 'language')
     ]
-    return render_template('gutenberg/book_details.html', record=record, fields=fields)
+    return render_template('gutenberg/book_details.html', record=record,
+                           fields=fields, epub_url=epub_url, htmlz_url=htmlz_url)
 
 
 @gutenberg.route('/text/<textId>/<int:textIndex>')
